/*==============================================================================
   DEMO.C :- RTF Code Generator Demo Program
   Sub Systems, Inc.
   Software License Agreement  (1998)
   ----------------------------------
   This is a demo program.  The program shows how you can utilize RTF Code
   Generator to create RTF codes.
===============================================================================*/
#define  STRICT

#include "windows.h"
#if defined (_WIN32)
   #if !defined(WIN32)
     #define WIN32
   #endif
#endif
#if !defined(WIN32)
  #include "print.h"
  #if defined (__TURBOC__)
    typedef unsigned short USHORT;     // missing in compiler
  #endif
#endif

#include "stdio.h"
#include "stdlib.h"
#include "fcntl.h"
#include "sys\types.h"
#include "sys\stat.h"
#include "io.h"
#include "string.h"

#include "commdlg.h"

/******************************************************************************
    WIN32 specific defines
*******************************************************************************/
#if defined (WIN32)
   #undef  huge
   #define huge
   #undef  FreeProcInstance
   #define FreeProcInstance(x)
   #define GET_WM_COMMAND_ID(wp,lp) LOWORD(wp)
   #define NOTIFY_MSG(wp,lp)        HIWORD(wp)
   #define farmalloc  malloc
   #define farrealloc realloc
   #define farfree    free
#else
   #define GET_WM_COMMAND_ID(wp,lp) (wp)
   #define NOTIFY_MSG(wp,lp)        HIWORD(lp)
#endif

#define true       TRUE
#define false      FALSE
#define null       NULL

#include "rcg.h"
#include "pdg.h"

#include "demo.h"

/****************************************************************************
   Main Windows Routine
****************************************************************************/
int PASCAL WinMain(HINSTANCE hInstance,HINSTANCE hPrevInstance,LPSTR lpCmdLine,int nCmdShow)
{
    MSG msg;
    int i;

    #ifndef WIN32
      int QueueLength=96;
      while (!SetMessageQueue(QueueLength) && (QueueLength-=8));
    #endif

    SetMessageQueue(96);   // needed for OLE2

    if (!hPrevInstance)
        if (!InitApplication(hInstance))
            return (FALSE);

    if (!InitInstance(hInstance, nCmdShow))
        return (FALSE);

    while (ProcessMsg && GetMessage(&msg, NULL, 0, 0)) {
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

    FreeProcInstance((FARPROC)lpDemoDlg);       // free the dialog box

    return (msg.wParam);
}

/****************************************************************************
   Initialize window data and register window class
****************************************************************************/

BOOL InitApplication(HINSTANCE hInstance)
{
    WNDCLASS  wc;

    // check if working on an mbcs operating system
    #if defined (WIN32)
       CPINFO cp;
       GetCPInfo(CP_ACP,&cp);
       mbcs=(cp.MaxCharSize>1);
    #else
       mbcs=GetSystemMetrics(SM_DBCSENABLED);
    #endif

    return TRUE;
}


/****************************************************************************
    Save instance handle and create main window
****************************************************************************/
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
    hInst = hInstance;

    lpDemoDlg = MakeProcInstance((FARPROC)DemoDlg, hInst);
    hDemoWnd=CreateDialog(hInst,"DemoDlg",NULL,(DLGPROC)lpDemoDlg);
    ShowWindow(hDemoWnd,SW_NORMAL);

    ProcessMsg=TRUE;

    return (TRUE);
}

/****************************************************************************
    DemoDlg:
    Display the dialog box.
****************************************************************************/
int FAR PASCAL _export DemoDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    int i,j,fct;
    BOOL result;
    RECT rect;

    switch (message) {
        case WM_INITDIALOG:
            // initialize the text input area
            GetWindowRect(GetDlgItem(hDlg,IDC_INPUT1_TEXT),&rect);
            InputBoxWidth=rect.right-rect.left;
            InputBoxHeight=rect.bottom-rect.top;
            DisableInputArea(hDlg);      // Disable the text input area

            SendDlgItemMessage(hDlg,IDC_AUTO_REFRESH, BM_SETCHECK,AutoRefresh, 0L);

            SetDlgItemText(hDlg,IDC_OUTPUT_FILE,"test.pdf");  // default output file

            CallFunction(hDlg,ID_NEW,FALSE);   // start the new document

            // insert the function names in the list box
            SetFunctionNames(hDlg);                // set function names in the list box

            return (FALSE);

        case WM_COMMAND:
            switch (GET_WM_COMMAND_ID(wParam,lParam)) {
                case IDC_LIST_BOX:              // execute a function
                   if (NOTIFY_MSG(wParam,lParam)==LBN_SELCHANGE) {     // enable/disable paste link
                      fct=(int)SendDlgItemMessage(hDlg,IDC_LIST_BOX,LB_GETCURSEL, 0,0L);
                      CallFunction(hDlg,fct,FALSE);   // execute a function
                   }
                   break;

                case IDC_AUTO_REFRESH:
                   AutoRefresh=(BOOL)SendDlgItemMessage(hDlg,IDC_AUTO_REFRESH,BM_GETCHECK, 0, 0L);
                   break;

                case IDOK:
                   CallFunction(hDlg,CurFct,TRUE);   // execute the current function
                   break;

                case IDC_PURCHASE:
                   ShellExecute(hDlg,"open","http://www.subsystems.com/pdgw.htm",NULL,NULL,0);

                   break;

                case IDC_GENERATE_PDF:               // generate pdf
                   GeneratePdf(hDlg);
                   break; 

                case IDCANCEL:
                   EndDialog(hDlg, FALSE);
                   ProcessMsg=FALSE;       // get out of the message loop
                   return (TRUE);
                default:
                   ;
            }
            break;
    }
    return (FALSE);
}

/******************************************************************************
    GeneratePdf:
    Generate PDF using the current document object
******************************************************************************/
BOOL GeneratePdf(HWND hDlg)
{
    BYTE OutFile[300];
    BOOL result=false;

    SetDlgItemText(hDlg,IDC_STATUS,"");

    GetDlgItemText(hDlg,IDC_OUTPUT_FILE,OutFile,sizeof(OutFile)-1);

    // check the file names
    if (lstrlen(OutFile)==0) {
       MessageBeep(0);
       SetFocus(GetDlgItem(hDlg,IDC_OUTPUT_FILE));
       return TRUE;
    } 

    SetDlgItemText(hDlg,IDC_STATUS,"Generating...");
                   
                   
    // do the conversion
    //PdgSetBoolProp(RepId,PGPROP_BATCH,TRUE);
    PdgSetBoolProp(RepId,PGPROP_EXACT_TEXT_PLACEMENT,TRUE);
    //PdgSetPageBorder(RepId,BRDRTYPE_DBL,20,720,0xFF);

    //PdgSetNumProp(RepId,PGPROP_PERM_FLAGS,PERM_PRINT);
    //PdgSetTextProp(RepId,PGPROP_OWNER_PASSWORD,"owner");

    if ((BOOL)SendDlgItemMessage(hDlg,IDC_USE_BUFFER,BM_GETCHECK,0,0L)) {  // read the file in the memory buffer and do the conversion
        HGLOBAL hMem;
        int OutSize;
                       
        if ((hMem=PdgGenerateReportBuffer(RepId,DocId,&OutSize))!=NULL) {
            if (WriteToFile(OutFile,hMem,OutSize)) { // write the memory to the output file   
               result=true;
            }
        }
    }
    else {      // convert files directly
        if (PdgGenerateReportFile(RepId,DocId,OutFile)) result=true;
    }
                   
    SetDlgItemText(hDlg,IDC_STATUS,result?"Done!":"Error!");
    if (result) ShellExecute(hDlg,"open",OutFile,NULL,NULL,SW_SHOW);   // invoke Acrobat to show the test file

    return true;
}
 
/******************************************************************************
    WriteToFile:
    Write the data to the disk file
 ******************************************************************************/
BOOL WriteToFile(LPBYTE OutFile, HGLOBAL hMem, int InSize)
{
    HANDLE hFile;
    OFSTRUCT OpenBuf;
    LPBYTE pMem;
    int BytesWritten;

    unlink(OutFile);
    if (INVALID_HANDLE_VALUE==(hFile=CreateFile(OutFile,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL))) {
       MessageBox(NULL,"Error opening the output file",NULL,MB_OK);
       return false;
    }

    pMem=GlobalLock(hMem);

   if (!WriteFile(hFile,pMem,InSize,&((DWORD)BytesWritten),NULL) 
           || BytesWritten!=InSize) {
       MessageBox(NULL,"Error writing to the output file",NULL,MB_OK);
       return false;
    } 

    GlobalUnlock(hMem);
    GlobalFree(hMem);

    CloseHandle(hFile);             // close file

    return TRUE;
} 

/******************************************************************************
    DisableInputArea:
    Disable the input area.
******************************************************************************/
BOOL DisableInputArea(HWND hDlg)
{
    SetDlgItemText(hDlg,IDC_INPUT1_TITLE,"");
    SetDlgItemText(hDlg,IDC_INPUT2_TITLE,"");
    SetDlgItemText(hDlg,IDC_INPUT1_TEXT,"");
    SetDlgItemText(hDlg,IDC_INPUT2_TEXT,"");

    // reset the input box sizes
    SetWindowPos(GetDlgItem(hDlg,IDC_INPUT1_TEXT),NULL,0,0,InputBoxWidth,InputBoxHeight,SWP_NOMOVE|SWP_NOZORDER|SWP_HIDEWINDOW);
    SetWindowPos(GetDlgItem(hDlg,IDC_INPUT2_TEXT),NULL,0,0,InputBoxWidth,InputBoxHeight,SWP_NOMOVE|SWP_NOZORDER|SWP_HIDEWINDOW);
    SetWindowPos(GetDlgItem(hDlg,IDOK),NULL,0,0,0,0,SWP_NOSIZE|SWP_NOMOVE|SWP_NOZORDER|SWP_HIDEWINDOW);

    EnableWindow(GetDlgItem(hDlg,IDC_INPUT1_TEXT),FALSE);
    EnableWindow(GetDlgItem(hDlg,IDC_INPUT2_TEXT),FALSE);
    EnableWindow(GetDlgItem(hDlg,IDOK),FALSE);

    return TRUE;
}

/******************************************************************************
    EnableInputArea:
    Enable the first input area.
******************************************************************************/
BOOL EnableInputArea(HWND hDlg, BOOL first, LPBYTE title,BOOL enlarge)
{
    HWND hWnd=GetDlgItem(hDlg,enlarge?IDC_INPUT1_TEXT:IDC_INPUT2_TEXT);

    SetDlgItemText(hDlg,enlarge?IDC_INPUT1_TITLE:IDC_INPUT2_TITLE,title);

    EnableWindow(hWnd,TRUE);
    SetWindowPos(hWnd,NULL,0,0,InputBoxWidth,enlarge?(3*InputBoxHeight):InputBoxHeight,SWP_NOMOVE|SWP_NOZORDER|SWP_SHOWWINDOW);
    SetFocus(hWnd);

    EnableWindow(GetDlgItem(hDlg,IDOK),TRUE);
    SetWindowPos(GetDlgItem(hDlg,IDOK),NULL,0,0,0,0,SWP_NOSIZE|SWP_NOMOVE|SWP_NOZORDER|SWP_SHOWWINDOW);

    return TRUE;
}

/******************************************************************************
    SetFunctionNames:
    Set all function names
******************************************************************************/
BOOL SetFunctionNames(HWND hDlg)
{
    SetFunctionName(hDlg,ID_NEW,"New Document");
    SetFunctionName(hDlg,ID_INIT_SECT,"Initialize Section");
    SetFunctionName(hDlg,ID_BEGIN_TEXT,"Begin Section Text");
    SetFunctionName(hDlg,ID_RESET_CHAR_PROP,"Reset Character Properties");
    SetFunctionName(hDlg,ID_SET_FONT,"Set Font...");
    SetFunctionName(hDlg,ID_CHAR_STYLE_ID,"Set Character Style Id");
    SetFunctionName(hDlg,ID_RESET_PARA_PROP,"Reset Paragraph Properties");
    SetFunctionName(hDlg,ID_PARA_CENTER,"Paragraph Centering");
    SetFunctionName(hDlg,ID_PARA_RIGHT,"Paragraph Right Justification");
    SetFunctionName(hDlg,ID_PARA_DOUBLE_SPACE,"Paragraph Double Spacing");
    SetFunctionName(hDlg,ID_PARA_BORDER,"Paragraph Border");
    SetFunctionName(hDlg,ID_PARA_INDENT,"Paragraph Indents");
    SetFunctionName(hDlg,ID_PARA_SPACE,"Paragraph Spacing");
    SetFunctionName(hDlg,ID_PARA_SHADE,"Paragraph Shading");
    SetFunctionName(hDlg,ID_PARA_BULLET,"Paragraph Bullet");
    SetFunctionName(hDlg,ID_PARA_NUMBERING,"Paragraph Numbering");
    SetFunctionName(hDlg,ID_PARA_TAB_STOP,"Paragraph Tab Stops");
    SetFunctionName(hDlg,ID_PARA_STYLE_ID,"Set Paragraph Style Id");
    SetFunctionName(hDlg,ID_INSERT_TEXT,"Insert Text...");
    SetFunctionName(hDlg,ID_INSERT_TABLE,"Insert Table");
    SetFunctionName(hDlg,ID_INSERT_PICT,"Insert a Picture file...");
    SetFunctionName(hDlg,ID_INSERT_LINK_PICT,"Insert a Linked Picture file...");
    SetFunctionName(hDlg,ID_PASTE_PICT,"Paste Picture");
    SetFunctionName(hDlg,ID_PAPER,"Set Paper Size");
    SetFunctionName(hDlg,ID_MARGIN,"Set Page Margin");
    SetFunctionName(hDlg,ID_SECT_INFO,"Set Section Information");
    SetFunctionName(hDlg,ID_BEGIN_HDR,"Begin Header Group");
    SetFunctionName(hDlg,ID_END_HDR,"End Header Group");
    SetFunctionName(hDlg,ID_BEGIN_FTR,"Begin Footer Group");
    SetFunctionName(hDlg,ID_END_FTR,"End Footer Group");
    SetFunctionName(hDlg,ID_BEGIN_STYLESHEET,"Begin Stylesheet");
    SetFunctionName(hDlg,ID_BEGIN_STYLEITEM,"Begin a Stylesheet Item");
    SetFunctionName(hDlg,ID_END_STYLEITEM,"End a Stylesheet Item");
    SetFunctionName(hDlg,ID_END_STYLESHEET,"End Stylesheet");
    SetFunctionName(hDlg,ID_BEGIN_FRAME,"Begin Frame Group");
    SetFunctionName(hDlg,ID_END_FRAME,"End Frame Group");
    SetFunctionName(hDlg,ID_BEGIN_TEXT_BOX,"Begin a Text Box Group");
    SetFunctionName(hDlg,ID_END_TEXT_BOX,"End Text Box Group");
    SetFunctionName(hDlg,ID_DRAW_RECT,"Draw a Rectangle");
    SetFunctionName(hDlg,ID_DRAW_LINE,"Draw a Line");
    SetFunctionName(hDlg,ID_PAGE_BREAK,"Insert a page break");
    SetFunctionName(hDlg,ID_SECT_BREAK,"Insert a section break");
    SetFunctionName(hDlg,ID_INSERT_PAGE_NO,"Insert Page Number");
    SetFunctionName(hDlg,ID_INSERT_MERGE_FIELD,"Insert a Mail/Merge Field");
    SetFunctionName(hDlg,ID_INSERT_TOC,"Insert a table-of-contents");
    SetFunctionName(hDlg,ID_INSERT_RAW,"Insert Raw RTF Code...");
    SetFunctionName(hDlg,ID_INSERT_FNOTE,"Insert footnote...");
    SetFunctionName(hDlg,ID_INSERT_UTEXT,"Insert Unicode Text");
    SetFunctionName(hDlg,ID_OTHER," ");

    return TRUE;
}


/******************************************************************************
    CallFunction:
    Call the specified API function.
******************************************************************************/
BOOL CallFunction(HWND hDlg,int fct, BOOL OnOK)
{
    BYTE string[RCG_MAX_WIDTH+1];

    CurFct=fct;                            // current function

    switch (fct) {
        case ID_NEW:                       // new document
           if (RepId) PdgEndSession(RepId);    // end the previous document
           RepId=PdgNewSession();              // new report session
           DocId=PdgNewDocObject(RepId);       // document id for this report
           break;

        case ID_BEGIN_STYLESHEET:          // begin the stylesheet group
           RcgBeginStyleSheet(DocId);break;

        case ID_BEGIN_STYLEITEM:           // begin a stylesheet item
           RcgBeginStyleItem(DocId,TRUE,StyleItemCount,"Normal");
           StyleItemCount++;
           break;

        case ID_END_STYLEITEM:              // end current stylesheet item
           RcgEndStyleItem(DocId);break;

        case ID_END_STYLESHEET:             // end current stylesheet group
           RcgEndStyleSheet(DocId);break;


        case ID_PAPER:                     // set paper size to 8.5 by 11 inches
           //RcgPaper(DocId,(int)(8.5*1440),(int)(11*1440),FALSE);   // letter size portrait
           RcgPaper(DocId,16834,11909,TRUE);   // A4 in landscape orientation
           break;

        case ID_MARGIN:                     // set 1 inche margin around
           RcgMargin(DocId,1440,1440,1440,1440,-1,-1);
           break;

        case ID_INIT_SECT:                 // initialize the section
           RcgInitSect(DocId);break;

        case ID_SECT_INFO:                 // set the initial page number at 30
           RcgSectInfo(DocId,1,0,TRUE,30);break;

        case ID_BEGIN_HDR:                 // begin the header group
           RcgBeginHdrFtr(DocId,RCG_HDR);break;

        case ID_END_HDR:                   // end the header group
           RcgEndHdrFtr(DocId);break;

        case ID_BEGIN_FTR:                 // begin the footer group
           RcgBeginHdrFtr(DocId,RCG_FTR);break;

        case ID_END_FTR:                   // end the footer group
           RcgEndHdrFtr(DocId);break;

        case ID_BEGIN_FRAME:               // begin the footer group
           RcgBeginFrame(DocId,TRUE,2000,2000,4000,4000);break;

        case ID_END_FRAME:                 // end the footer group
           RcgEndFrame(DocId);break;

        case ID_BEGIN_TEXT_BOX:            // begin the text box group
           RcgBeginTextBox(DocId,TRUE,1500,1500,2000,2000,-1,0xFFFFFF,0,20,FALSE);
           break;

        case ID_END_TEXT_BOX:              // end the text box  group
           RcgEndTextBox(DocId);break;

        case ID_DRAW_RECT:                 // draw a rectangle
           RcgDrawRect(DocId,TRUE,3500,3500,2000,2000,-1,0xFFFF,0,20,FALSE);
           break;

        case ID_DRAW_LINE:                 // draw a line
           RcgDrawLine(DocId,TRUE,2500,2500,3500,3500,-1,0xFF,60);
           break;

        case ID_PAGE_BREAK:                 // page break
           RcgInsertBreak(DocId,RCG_BREAK_PAGE);break;

        case ID_SECT_BREAK:                 // page break
           RcgInsertBreak(DocId,RCG_BREAK_SECT);break;

        case ID_BEGIN_TEXT:                // text begins
           RcgBeginSectText(DocId);break;

        case ID_RESET_CHAR_PROP:           // reset the character properties
           RcgResetTextProp(DocId);break;

        case ID_RESET_PARA_PROP:           // reset the paragraph properties
           RcgResetParaProp(DocId);break;

        case ID_SET_FONT:
           SetFont(hDlg);break;

        case ID_CHAR_STYLE_ID:             // specify the character style id
           RcgCharStyleId(DocId,1);break;

        case ID_PARA_CENTER:               // specify the paragraph centering
           RcgParaFlags(DocId,RCG_CENTER);break;

        case ID_PARA_RIGHT:               // specify the paragraph right justification
           RcgParaFlags(DocId,RCG_RIGHT);break;

        case ID_PARA_DOUBLE_SPACE:        // specify the paragraph double spacing
           RcgParaFlags(DocId,RCG_DOUBLE_SPACE);break;

        case ID_PARA_BORDER:              // Apply single width paragraph border
           RcgParaFlags(DocId,RCG_PARA_BOX);break;

        case ID_PARA_INDENT:              // Apply paragraph indentation
           RcgParaIndent(DocId,1440,1440,1440);break;

        case ID_PARA_SPACE:               // Apply paragraph spacing
           RcgParaSpace(DocId,720,720,400);break;

        case ID_PARA_SHADE:               // Apply paragraph shading
           RcgParaShadeColor(DocId,30,0);break;

        case ID_PARA_BULLET:               // Apply paragraph bullet
           RcgParaIndent(DocId,720,720,-720);
           //RcgParaBullet(DocId,TRUE,BLT_ROUND,0,1,NULL,NULL,0);
           RcgParaBullet(DocId,FALSE,NBR_DEC,0,1,NULL,".",0);
           break;

        case ID_PARA_NUMBERING:           // Apply paragraph Numbering
           RcgParaIndent(DocId,720,720,-720);
           RcgParaBullet(DocId,FALSE,NBR_DEC,0,1,"(",")",0);
           break;

        case ID_PARA_TAB_STOP:            // create paragraph tab stops
           RcgParaTabStop(DocId,0,1440,0);      // at one inch
           RcgParaTabStop(DocId,0,3*1440,0);    // at 3 inch
           RcgParaTabStop(DocId,TAB_RIGHT,5*1440,TAB_DOT);  // right tab with dot leader at 5 inches
           break;

        case ID_PARA_STYLE_ID:             // specify the paragraph style id
           RcgParaStyleId(DocId,1);break;

        case ID_INSERT_TEXT:
           if (OnOK) {
              GetDlgItemText(hDlg,IDC_INPUT1_TEXT,string,sizeof(string)-1);
              RcgInsertText(DocId,string);      // insert the text into the rtf buffer
              DisableInputArea(hDlg);
           }
           else EnableInputArea(hDlg,TRUE,"Please enter the text here and click OK",TRUE);
           break;

        case ID_INSERT_UTEXT:
           {
              BYTE text[]="Text in Unicode";
              int i,len=lstrlen(text);
              WORD UText[100];

              for (i=0;i<=len;i++) UText[i]=text[i];
              RcgInsertUText(DocId,UText);      // insert the text into the rtf buffer
           }
           break;

        case ID_INSERT_RAW:
           if (OnOK) {
              GetDlgItemText(hDlg,IDC_INPUT1_TEXT,string,sizeof(string)-1);
              RcgInsertRaw(DocId,string);      // insert the text into the rtf buffer
              DisableInputArea(hDlg);
           }
           else EnableInputArea(hDlg,TRUE,"Please enter raw RTF code here and click OK",TRUE);
           break;

        case ID_INSERT_FNOTE:
           if (OnOK) {
              GetDlgItemText(hDlg,IDC_INPUT1_TEXT,string,sizeof(string)-1);
              RcgInsertFootnote(DocId,NULL,string);      // insert the text into the rtf buffer
              DisableInputArea(hDlg);
           }
           else EnableInputArea(hDlg,TRUE,"Please enter footnote text here and click OK",TRUE);
           break;

        case ID_INSERT_TABLE:
           InsertTable();
           break;

        case ID_INSERT_PICT:
           lstrcpy(string,"*.bmp");
           if (0==GetFileName(TRUE,string,"bmp","Bitmap File(*.BMP)|*.BMP|Windows Metafile(*.WMF)|*.WMF|Enhanced Metafile(*.EMF)|*.EMF|")) break;
           RcgInsertPictFile(DocId,string,0,0,FALSE);      // save the code the file
           break;

        case ID_INSERT_LINK_PICT:
           lstrcpy(string,"*.bmp");
           if (0==GetFileName(TRUE,string,"bmp","Bitmap File(*.BMP)|*.BMP|Windows Metafile(*.WMF)|*.WMF|")) break;
           RcgInsertPictFile(DocId,string,0,0,TRUE);      // save the code the file
           break;

        case ID_PASTE_PICT:
           RcgPastePicture(DocId,0,NULL,0,0);
           break;

        case ID_INSERT_PAGE_NO:
           RcgInsertText(DocId,"Page# ");      // page number label
           RcgInsertPageField(DocId,TRUE);     // insert page number field
           RcgInsertText(DocId," Total Pages: ");
           RcgInsertPageField(DocId,FALSE);    // insert page count field
           break;

        case ID_INSERT_MERGE_FIELD:
           RcgInsertMergeField(DocId,"FieldName","FieldData");
           break;

        case ID_INSERT_TOC:
           RcgInsertTocField(DocId,1,3);
           break;

        case ID_OTHER:                        // this code is for miscellaneous testing
           DemoOther();
           break;


    }

    if (AutoRefresh && DocId) ShowRtfCode(hDlg);   // refresh the code text box

    return TRUE;
}

/******************************************************************************
    DemoOther:
    Test code.
******************************************************************************/
BOOL DemoOther()
{
  int RepId,DocId,size=0;
  LPBYTE pMem;

  RepId=PdgNewSession(); // start a reporting session

  DocId=PdgNewDocObject(RepId); // begin new document, and return 
                           //  a  document handle
  
  //PdgSetBoolProp(RepId,PGPROP_EMBED_FONTS,true);
  PdgSetBoolProp(RepId,PGPROP_EXACT_TEXT_PLACEMENT,true);

  RcgInitSect(DocId); // begin first section
                                  
  pMem=PdgFileToMemPtr("k:\\rpc\\test.rtf",&size);                                  
  pMem[size]=0;

  RcgInsertRaw(DocId,pMem);
    
  RcgSaveFile(DocId,"test.rtf");     // debug code to see intermediate rtf file
    
  PdgGenerateReportFile(RepId,DocId,"test.pdf"); // generate PDF

  PdgEndSession(RepId); // free the current report handle

  ShellExecute(NULL,"open","test.pdf",NULL,NULL,SW_SHOW);
    
    return TRUE;
}
 
/******************************************************************************
    SetFunctionName:
    Set a function name in the selection list box.
******************************************************************************/
BOOL SetFunctionName(HWND hDlg,int id, LPBYTE name)
{
    return (BOOL) SendDlgItemMessage(hDlg,IDC_LIST_BOX,LB_INSERTSTRING,(WPARAM)id,(LPARAM)name);
}


/******************************************************************************
    ShowRtfCode:
    Show the rtf code so far generated.
******************************************************************************/
BOOL ShowRtfCode(HWND hDlg)
{
    long len;
    HGLOBAL hCode=RcgGetData(DocId,&len);
    LPBYTE pCode;
    int lines;
    HWND hEdit=GetDlgItem(hDlg,IDC_RTF);

    if (!hCode) return FALSE;

    pCode=GlobalLock(hCode);
    SetDlgItemText(hDlg,IDC_RTF,pCode);
    GlobalUnlock(hCode);
    GlobalFree(hCode);

    // position at the end of the buffer
    lines=(int)SendMessage(hEdit,EM_GETLINECOUNT,0,0L);
    SendMessage(hEdit,EM_LINESCROLL,0,MAKELPARAM(lines-20,0));  // show last 20 lines in the control

    return TRUE;
}

/******************************************************************************
    SetFont:
    Show the common dialog to accept new font/color for the text
******************************************************************************/
BOOL SetFont(HWND hDlg)
{
    int i,ResY,NewFont,SaveHeight,SaveWidth,GroupItem;
    CHOOSEFONT cFont;
    LOGFONT lFont;
    int font;

    // Fill the lFont structure
    memset(&lFont,0,sizeof(lFont));
    ResY=GetDeviceCaps(GetDC(hDlg),LOGPIXELSY);
    lFont.lfHeight=-(int)(((long)CurPointSize*ResY)/72);
    lFont.lfWeight=CurStyle&RCG_BOLD ? 700 : 400;
    lFont.lfItalic=(BYTE)(CurStyle&RCG_ITALIC ? TRUE : FALSE);
    lFont.lfUnderline=(BYTE)(CurStyle&RCG_ULINE  ? TRUE : FALSE);
    lFont.lfStrikeOut=(BYTE)(CurStyle&RCG_STRIKE ? TRUE : FALSE);
    lFont.lfCharSet=ANSI_CHARSET;
    lFont.lfOutPrecision=OUT_DEFAULT_PRECIS;
    lFont.lfClipPrecision=CLIP_DEFAULT_PRECIS;
    lFont.lfQuality=DRAFT_QUALITY;
    lFont.lfPitchAndFamily=DEFAULT_PITCH|FF_DONTCARE;
    lstrcpy(lFont.lfFaceName,CurTypeface);

    // fill the CHOOSEFONT structure
    memset(&cFont,0,sizeof(CHOOSEFONT));
    cFont.lStructSize=sizeof(CHOOSEFONT);
    cFont.hwndOwner=hDlg;
    cFont.hDC=0;                    // print device context
    cFont.lpLogFont=&lFont;
    cFont.Flags=CF_BOTH|CF_EFFECTS|CF_INITTOLOGFONTSTRUCT;
    cFont.rgbColors=CurColor;

    if (!ChooseFont(&cFont)) return FALSE;

    // Extract the chosen properties
    lstrcpy(CurTypeface,lFont.lfFaceName);
    CurPointSize=(int)(((long)lFont.lfHeight*(long)72)/ResY);
    if (CurPointSize<0) CurPointSize=-CurPointSize;
    CurColor=cFont.rgbColors;

    CurStyle=0;

    if (lFont.lfWeight>400) CurStyle|=RCG_BOLD;
    if (lFont.lfItalic)     CurStyle|=RCG_ITALIC;
    if (lFont.lfUnderline)  CurStyle|=RCG_ULINE;
    if (lFont.lfStrikeOut)  CurStyle|=RCG_STRIKE;

    RcgTextFont(DocId,CurTypeface,CurPointSize,CurStyle,TRUE);

    RcgTextColor(DocId,CurColor,TRUE);

    return TRUE;
}

/******************************************************************************
    InsertTable:
    Insert a table. This function creates a table of 2 columns and 3 rows.
    The first row contains the header.  The table list the name and price of
    a couple products.
******************************************************************************/
BOOL InsertTable()
{
    // Create the header row.
    //RcgBeginTableRow(DocId,0,0,RCG_CENTER,FALSE,0);
    RcgBeginTableRow2(DocId,0,0,RCG_CENTER,FALSE,200,200,200,200);
    
    RcgNextCellInfo(DocId,3*1440,0,30,0xFFFFFF,20,40,20,40,0);
    RcgNextCellInfo(DocId,1440,0,30,0xFFFFFF,40,20,20,40,0);

    // row 1 col 1
    RcgBeginCellText(DocId);
    RcgResetTextProp(DocId);
    RcgTextFont(DocId,"",0,RCG_BOLD,TRUE);
    RcgInsertText(DocId,"Products");
    RcgEndCellText(DocId);

    // row 1 col 2
    RcgBeginCellText(DocId);
    RcgResetTextProp(DocId);
    RcgTextFont(DocId,"",0,RCG_BOLD,TRUE);
    RcgInsertText(DocId,"Price");
    RcgEndCellText(DocId);

    RcgEndTableRow(DocId);

    // create the second row
    RcgBeginTableRow(DocId,0,0,RCG_CENTER,FALSE,0);
    RcgNextCellInfo(DocId,3*1440,0,0,0xFFFFFF,20,40,40,20,0);
    RcgNextCellInfo(DocId,1440,0,0,0xFFFFFF,40,20,40,20,0);

    // row 2 col 1
    RcgBeginCellText(DocId);
    RcgResetTextProp(DocId);
    RcgInsertText(DocId,"TE Developer's Kit");
    RcgEndCellText(DocId);

    // row 2 col 2
    RcgBeginCellText(DocId);
    RcgResetTextProp(DocId);
    RcgInsertText(DocId,"$439.00");
    RcgEndCellText(DocId);

    RcgEndTableRow(DocId);

    // create the third row
    RcgBeginTableRow(DocId,0,0,RCG_CENTER,FALSE,0);
    RcgNextCellInfo(DocId,3*1440,0,0,0xFFFFFF,20,40,20,20,0);
    RcgNextCellInfo(DocId,1440,0,0,0xFFFFFF,40,20,20,20,0);

    // row 2 col 1
    RcgBeginCellText(DocId);
    RcgResetTextProp(DocId);
    RcgInsertText(DocId,"ReportEase Plus");
    RcgEndCellText(DocId);

    // row 2 col 2
    RcgBeginCellText(DocId);
    RcgResetTextProp(DocId);
    RcgInsertText(DocId,"$459.00");
    RcgEndCellText(DocId);

    RcgEndTableRow(DocId);

    return TRUE;
}

/******************************************************************************
                             Helper Routines
*******************************************************************************/

/******************************************************************************
    GetFileName:
    This function gets the file name from the user.  The first argument is
    TRUE for 'open' dialog box, and FALSE for the 'Save As' dialog box.
    The second argument can be used to specify the initial file name and
    get the file selection from the user.  The third argument (filter) must
    be '|' for delimters for wild card pairs.
    The function return the file type index indicating the index into the filter.
    The value is 0 when an error occurs.
******************************************************************************/
int GetFileName(BOOL open,LPBYTE file,LPBYTE DefExt,LPBYTE filter)
{
    int    i,len,result;
    OPENFILENAME ofn;
    MSG msg;

    // preprare to get the bitmap file name from the user
    memset(&ofn,0,sizeof(OPENFILENAME));        // initialize
    ofn.lStructSize=sizeof(OPENFILENAME);
    ofn.hwndOwner=hDemoWnd;
    len=lstrlen(filter);
    for (i=0;i<len;i++) if (filter[i]=='|') filter[i]=0;
    ofn.lpstrFilter=filter;
    if (len) ofn.nFilterIndex=1;
    else     ofn.nFilterIndex=0;
    ofn.lpstrFile=file;
    ofn.lpstrDefExt=DefExt;
    ofn.nMaxFile=128;
    ofn.Flags=OFN_SHOWHELP|OFN_HIDEREADONLY|OFN_NOCHANGEDIR;

    // get the file name from the user
    if (open) result=GetOpenFileName(&ofn);
    else {                              // save file
       ofn.Flags|=OFN_OVERWRITEPROMPT;
       result=GetSaveFileName(&ofn);
    }

    if (result) {                       // get the filter index
       result=(int)(ofn.nFilterIndex);  // base 1
    }

    SetFocus(hDemoWnd);

    // discard any mouse messages left from the above call
    while (PeekMessage(&msg,hDemoWnd,WM_MOUSEFIRST,WM_MOUSELAST,PM_REMOVE|PM_NOYIELD));;

    return result;
}


/******************************************************************************
     AssignIntValue:
     Extract an integer value from the dialog item.  Display error if a non-
     numeric value is encountered and return a false result.
*******************************************************************************/
BOOL AssignIntValue(HWND hWin,int ControlId,LPINT pInt)
{
    BOOL ErrorFlag;

    *pInt=GetDlgItemInt(hWin,ControlId,&ErrorFlag,(BOOL) 0);
    if (!ErrorFlag) {
       MessageBox(hWin,"Must Enter a Valid Number",NULL,MB_OK);
       SetFocus(GetDlgItem(hWin,ControlId));
       return (FALSE);
    }
    return TRUE;
}

/******************************************************************************
     AssignLongValue:
     Extract a long value from the dialog item.  Display error if a non-
     numeric value is encountered and return a false result.
*******************************************************************************/
BOOL AssignLongValue(HWND hWin,int ControlId,LPLONG pLong)
{
    char string[20];
    int i;

    GetDlgItemText(hWin,ControlId,string,sizeof(string)-1);  // get the number

    for (i=0;i<strlen(string);i++) {  // check for non numeric characters
        if (string[i]=='-') continue;
        if (string[i]<'0' || string[i]>'9') {
           MessageBox(hWin,"Must Enter a Valid Number",NULL,MB_OK);
           SetFocus(GetDlgItem(hWin,ControlId));
           return (FALSE);
        }
    }

    (*pLong)=atol(string);       // convert to long

    return TRUE;
}

/******************************************************************************
     SetDemoLong:
     Set a long value to a dialog field.
*******************************************************************************/
BOOL SetDemoLong(HWND hDlg,int ControlId,long value)
{
    char string[20];

    wsprintf(string,"%ld",value);
    SetDlgItemText(hDlg,ControlId,string);

    return TRUE;
}

/******************************************************************************
     GetDemoLong:
     Gety a long value from a dialog field.
*******************************************************************************/
BOOL GetDemoLong(HWND hDlg,int ControlId,long *value)
{
    BYTE string[20];

    GetDlgItemText(hDlg,ControlId,string,19);
    (*value)=atol(string);

    return TRUE;
}

/******************************************************************************
    OurPrintf:
    This routine formats and display a given set of arguments.  The format
    is given by the first argument.  The argument 2 to n contain the
    arguments for the specified format.  The function uses MessageBox to
    display the formatted string.
******************************************************************************/
int OurPrintf(LPSTR fmt,...)
{
    LPSTR ArgStart;
    char string[RCG_MAX_WIDTH+2];
    HWND hWnd;

    ArgStart=(LPSTR) &fmt;  // pointer to first argument
    ArgStart=&ArgStart[4];  // go past the first argument
    wvsprintf(string,fmt,ArgStart);

    if (NULL!=(hWnd=FindWindow("DBWin",NULL))) { // debug window open
        if (NULL!=(hWnd=GetWindow(hWnd,GW_CHILD))) {    // get the edit window
           int len=lstrlen(string);
           string[len]=0xd;
           string[len+1]=0xa;
           string[len+2]=0;
           #if defined (WIN32)
              SendMessage(hWnd,EM_SETSEL,32767,32767);  // show the last line
              SendMessage(hWnd,EM_REPLACESEL,0,(DWORD)(LPSTR)string);
              SendMessage(hWnd,EM_SCROLLCARET,0,0);     // scroll caret into view
           #else
              SendMessage(hWnd,EM_SETSEL,0,MAKELPARAM(32767,32767));  // show the last line
              SendMessage(hWnd,EM_REPLACESEL,0,(DWORD)(LPSTR)string);
              SendMessage(hWnd,EM_SETSEL,0,MAKELPARAM(32767,32767));  // show the last line
           #endif
        }
    }
    else MessageBox(NULL,string,NULL,MB_OK);
    return TRUE;
}


/****************************************************************************
                      DEMO DIALOG PROCEDURES
****************************************************************************/


/****************************************************************************
    SetGuiFont:
    Set the gui font for a text input box.
****************************************************************************/
SetGuiFont(HWND hDlg,int id)
{
    HFONT hFont;

    if (!mbcs) return TRUE;

    #if defined(WIN32)
      hFont=GetStockObject(DEFAULT_GUI_FONT);
    #else
      hFont=GetStockObject(SYSTEM_FONT);
    #endif

    if (hFont) {
       SendMessage(hDlg,WM_SETFONT,(WPARAM)hFont,0L);
       SendDlgItemMessage(hDlg,id,WM_SETFONT,(WPARAM)hFont,0L);
    }
    return TRUE;
}
